--
-- Overrunbreak
-- Specialization for a overrun break
--
-- @author  Stefan Maurus	
-- @date  10/11/14

-- Copyright © Stefan Maurus, www.stefanmaurus.de

--[[
	<overrunbreak speed="FLOAT" movingTools="0 1 2 3 .." movingParts="0 1 2 3 ..">	
		<part index="INDEX" startTrans="" endTrans="" startRot="" endRot="" startScale="" endScale=""/>
		<!--more parts-->
	</overrunbreak>
]]

Overrunbreak = {};

function Overrunbreak.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Cylindered, specializations);
end;

function Overrunbreak:load(xmlFile)
	self.setOverRunBreakAnimCurves = SpecializationUtil.callSpecializationsFunction("setOverRunBreakAnimCurves");

	self.overrunbreak = {};
	self.overrunbreak.speed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.overrunbreak#speed"), 1);
	self.overrunbreak.movingTools = Utils.getVectorNFromString(getXMLString(xmlFile, "vehicle.overrunbreak#movingTools"));
	self.overrunbreak.movingParts = Utils.getVectorNFromString(getXMLString(xmlFile, "vehicle.overrunbreak#movingParts"));

	self.overrunbreak.currentTime = 0;
	
	local i = 0;
	self.overrunbreak.animCurves = {};
	while true do 
		local key = string.format("vehicle.overrunbreak.part(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local index = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
		
		local animCurve = AnimCurve:new(linearInterpolatorTransRotScale);
		
		local xST, yST, zST = Utils.getVectorFromString(getXMLString(xmlFile, key.."#startTrans"));
		if xST == nil or yST == nil or zST == nil then
			xST, yST, zST = getTranslation(index);
		end;
		
		local xET, yET, zET = Utils.getVectorFromString(getXMLString(xmlFile, key.."#endTrans"));
		if xET == nil or yET == nil or zET == nil then
			xET, yET, zET = getTranslation(index);
		end;
		
		local xSR, ySR, zSR = Utils.getVectorFromString(getXMLString(xmlFile, key.."#startRot"));
		if xSR == nil or ySR == nil or zSR == nil then
			xSR, ySR, zSR = getRotation(index);
		else
			xSR = math.rad(xSR);
			ySR = math.rad(ySR);
			zSR = math.rad(zSR);			
		end;
		
		local xER, yER, zER = Utils.getVectorFromString(getXMLString(xmlFile, key.."#endRot"));
		if xER == nil or yER == nil or zER == nil then
			xER, yER, zER = getRotation(index);
		else
			xER = math.rad(xER);
			yER = math.rad(yER);
			zER = math.rad(zER);			
		end;

		local xSS, ySS, zSS = Utils.getVectorFromString(getXMLString(xmlFile, key.."#startScale"));
		if xSS == nil or ySS == nil or zSS == nil then
			xSS, ySS, zSS = getScale(index);
		end;	

		local xES, yES, zES = Utils.getVectorFromString(getXMLString(xmlFile, key.."#endScale"));
		if xES == nil or yES == nil or zES == nil then
			xES, yES, zES = getScale(index);
		end;			
		
		animCurve:addKeyframe({x=xST, y=yST, z=zST, rx=xSR, ry=ySR, rz=zSR, sx=xSS, sy=ySS, sz=zSS, time=0});
		animCurve:addKeyframe({x=xET, y=yET, z=zET, rx=xER, ry=yER, rz=zER, sx=xES, sy=yES, sz=zES, time=1});
		
		table.insert(self.overrunbreak.animCurves, {index=index, animCurve=animCurve});
		i = i + 1;		
	end;
	
	self.brakeForceFix = self.brakeForce;
	
	self:setOverRunBreakAnimCurves(0);
end;

function Overrunbreak:delete()
end;

function Overrunbreak:readStream(streamId, connection)
end;

function Overrunbreak:writeStream(streamId, connection) 
end;

function Overrunbreak:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Overrunbreak:keyEvent(unicode, sym, modifier, isDown)
end;

function Overrunbreak:update(dt)
	local axisForward = 0;
	axisForward = InputBinding.getDigitalInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE);
	if InputBinding.isAxisZero(axisForward) then
		axisForward = InputBinding.getAnalogInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE); --if no move key is pressed check gamepad (analog axis)
	end;
	
	local speedKMH = self.lastSpeed*3600;
	local movingForward = self.movingDirection > 0;
	local breaking = axisForward > 0;

	if movingForward and breaking and speedKMH > 4 and self.overrunbreak.currentTime < 1 then
		self:setOverRunBreakAnimCurves(self.overrunbreak.currentTime+self.overrunbreak.speed*speedKMH);
	elseif movingForward and not breaking and speedKMH > 1 and self.overrunbreak.currentTime > 0 then
		self:setOverRunBreakAnimCurves(self.overrunbreak.currentTime-self.overrunbreak.speed*speedKMH);
	end;
	
	if not movingForward and speedKMH > 0.2 then
		self:setOverRunBreakAnimCurves(self.overrunbreak.currentTime+self.overrunbreak.speed*speedKMH*5);
	end;
end;

function Overrunbreak:updateTick(dt)	
end;

function Overrunbreak:draw()
end;

function Overrunbreak:onAttach()
end;

function Overrunbreak:onDetach()	
end; 

function Overrunbreak:setOverRunBreakAnimCurves(t)
	for _, part in pairs(self.overrunbreak.animCurves) do
		local v = {part.animCurve:get(t)};
		setTranslation(part.index, v[1], v[2], v[3]);
		setRotation(part.index, v[4], v[5], v[6]);
		setScale(part.index, v[7], v[8], v[9]);
	end;
	
	self.overrunbreak.currentTime = math.max(math.min(1,t), 0);

	for i, part in ipairs(self.movingParts) do
		for j=0,table.getn(self.overrunbreak.movingParts) do
			if i == self.overrunbreak.movingParts[j] then
				Cylindered.updateMovingPart(self, part, true);
			end;
		end;
	end;
	
	for i, tool in ipairs(self.movingTools) do
		for j=0,table.getn(self.overrunbreak.movingTools) do
			if i == self.overrunbreak.movingTools[j] then
				Cylindered.setDirty(self, tool);
			end;
		end;
	end;	

	self.brakeForce = math.max(self.brakeForceFix * t, self.brakeForceFix*0.5);
end;